
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

## This concern has +follow+ and +unfollow+ controller actions
## When you use it, be sure that +set_followable+ method is implemented in your controller, otherwise an fatal error will raise.
module FollowableController
  extend ActiveSupport::Concern

  included do
    before_action :authenticate_user!, only: [:follow, :unfollow, :follow_toggle]
  end

  # POST /v1/users/1/follow
  # POST /v1/users/1/follow.json
  def follow
    if !current_user.following?(followable)
      current_user.follow(followable)
      render status: :created
    else
      render status: :forbidden
    end
  end

  # DELETE /v1/users/1/unfollow
  # DELETE /v1/users/1/unfollow.json
  def unfollow
    if current_user.following?(followable)
      current_user.unfollow(followable)
      render status: :ok
    else
      render status: :forbidden
    end
  end

  # PUT /v1/users/1/follow
  # PUT /v1/users/1/follow.json
  def follow_toggle
    if !current_user.following?(followable)
      current_user.follow(followable)
      render status: :created
    elsif current_user.following?(followable)
      current_user.unfollow(followable)
      render status: :ok
    else
      render status: :forbidden
    end
  end
end
