
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

module Paginator
  extend ActiveSupport::Concern

  def paginate(model)
    model = params_actions(model)
    total_count model
    return model.limit(limit).offset(offset) if model.respond_to?('limit')
    return model[offset..limit]
  end

  private

    def limit
      return params[:limit].to_i if !params[:limit].blank?
      return params[:results_per_page].to_i if !params[:results_per_page].blank?
      return 12
    end


    def offset
      return params[:offset].to_i if !params[:offset].blank?
      return params[:page].to_i*limit if !params[:page].blank?
      return 0
    end


    def params_actions(model)

      # filtering
      if !params[:filter].blank?
        (JSON.parse params[:filter]).each do |key, value|
          if(model.columns_hash[key].type == :integer)
            model = model.where("#{key}": value)
          else
            model = model.where("#{key} ILIKE  '%'|| ? || '%'", value)
          end
        end
      end

      # ordering
      to_sort = JSON.parse params[:sort] if !params[:sort].blank?
      model = model.order("\"#{model.table_name}\".\"#{to_sort[0]}\" #{to_sort[1]}") if !to_sort.blank?

      return model
    end

    def total_count(model)
      headers['X-Total-Count'] = model.count
    end

end
