
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::AchievementsController < ApplicationController
  include ::Paginator

  before_action :set_achievement, only: [:show, :update, :destroy]
  before_action :authenticate_user!, except: [:index, :show]
  before_action :authorize!, only: [:update, :destroy]

  # GET /v1/achievements
  def index
    achievements = paginate Achievement
    render json: achievements
  end

  # GET /v1/achievements/1
  def show
    render json: @achievement
  end

  # POST /v1/achievements
  def create
    @achievement = Achievement.new(achievement_params)
    authorize @achievement

    if @achievement.save
      @achievement.add_requirements(extra_params[:requirements])
      render json: @achievement, status: :created
    else
      render json: @achievement.errors, status: :unprocessable_entity
    end
  end

  # PUT/PATCH /v1/achievements/1
  def update
    if @achievement.update(achievement_params)
      @achievement.update_requirements(extra_params[:requirements])
      render json: @achievement, status: :ok
    else
      render json: @achievement.errors, status: :unprocessable_entity
    end
  end

  # DELETE /v1/achievements/1
  def destroy
    if @achievement.update(state: 2)
      render status: :ok, json: @achievement
    else
      render json: @achievement.errors, status: :unprocessable_entity
    end
  end

  private

  def set_achievement
    @achievement ||= Achievement.find_by_id(params[:id])
    if @achievement.blank?
      render status: :not_found
    end
  end

  def achievement_params
    params.require(:achievement).permit(:name, :description, :reward_experience, :reward_points, :state, :repeatable, :resettable)
  end

  def extra_params
    return {} if params[:achievement].nil?
    params[:achievement].permit(requirements: [])
  end

  def authorize!
    authorize @achievement
  end
end
