class V1::ComplaintVotesController < ApplicationController
    include ::DeletedObjectsController
    include ::Paginator
    before_action :set_complaint_vote, only: [:show, :update, :destroy]

    # GET /v1/complaint_votes
    def index
        complaint_votes = paginate ComplaintVote
        render json: complaint_votes
    end

    # GET /v1/complaint_votes/{id}
    def show
        render json: @complaint_vote
    end

    # POST /v1/complaint_votes
    def create
        @complaint_vote = ComplaintVote.where(complainable_id: params[:complainable_id]).first

        # if object does not exists create
        if @complaint_vote.blank?
            @complaint_vote = ComplaintVote.new(complaint_vote_params)

            if @complaint_vote.save
                render json: @complaint_vote, status: :created
            else
                render json: @complaint_vote.errors, status: :unprocessable_entity
            end
        # if object already exists update the pros and cons votes by addding the new vote on the current votes 
        else
            vote_pros = @complaint_vote.pros + params[:pros]
            vote_cons = @complaint_vote.cons + params[:cons]

            @complaint_vote.update(pros: vote_pros, cons: vote_cons)
            
            verify_votes(@complaint_vote)
        end
    end

    # Set the params required for creating a complaint vote
    def complaint_vote_params
        params.require(:complaint_vote).permit(:complainable_type, :complainable_id, :pros, :cons)
    end

    # PUT/PATCH /v1/complaint_votes/{id}
    def update
        if @complaint_vote.update(complaint_vote_params)
            render json: @complaint_vote
        else 
            render json: @complaint_vote.errors, status: :unprocessable_entity
        end
    end

    # DELETE /v1/complaint_votes/{id}
    def destroy
        @complaint_vote.destroy
        response = { 'status': 'deleted' }
        render status: :ok, json: response
    end

    def set_complaint_vote
        @complaint_vote = ComplaintVote.where(id: params[:id]).first
        
        render status: :not_found if @complaint_vote.blank?
        
        @complaint_vote
    end
    
    # if the difference between the pros and cons votes is more than 2, destroy the learning object  
    def verify_votes(complaint_vote)
        if complaint_vote.pros - complaint_vote.cons > 2
            LearningObject.find_by_id(complaint_vote.complainable_id).destroy
            Complaint.find_by_complainable_id(complaint_vote.complainable_id).destroy
            ComplaintVote.find_by_complainable_id(complaint_vote.complainable_id).destroy
        elsif complaint_vote.cons - complaint_vote.pros > 2
            @current_complaint = Complaint.where(complainable_id: @complaint_vote.complainable_id).first
            @current_complaint.update(deleted_at: Time.now)
            ComplaintVote.find_by_complainable_id(complaint_vote.complainable_id).destroy
        end
    end

end