
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::ComplaintsController < ApplicationController
  include ::DeletedObjectsController
  include ::Paginator


  before_action :set_complaint, only: [:show, :show_related , :reject, :accept]
  before_action :authenticate_user!, only: [:index, :show, :show_related, :create, :reject, :accept, :treat_related]

  # GET v1/complaints
  # GET v1/complaints.json
  def index
    complaints = paginate Complaint
    render json: complaints
  end

  # GET /v1/complaints/1
  # GET /v1/complaints/1.json
  def show
    render :json => @complaint, :include => {:complaint_reason => {:except => [:updated_at, :created_at]}}
  end

  # GET /v1/complaints/1/show_related
  # GET /v1/complaints/1/show_related.json
  def show_related
    response = Complaint.where(complainable: @complaint.complainable, state: "complained")
    render status: :ok, json: response , :include => {:complaint_reason => {:except => [:updated_at, :created_at]}}
  end

  
  # POST v1/complaints
  # POST v1/complaints.json
  def create
    @complaint = Complaint.new(complaint_creation_params)
    
    if @complaint.save
      ComplaintsMailer.new_complaint_received(@complaint, @current_user).deliver_now
      
      @complaint.complainable.treat_complaintment
      
      render json: @complaint, status: :created
    else
      render json: @complaint.errors, status: :unprocessable_entity
    end
  end

  # POST v1/complaints/1/reject
  # POST v1/complaints/1/reject.json
  def reject
    @complaint.rejected!
    @complaint.complainable.complaint_reject(complaint_answer_params[:option])
    response = { 'status': 'complaint rejected' }
    render status: :ok, json: response
  end
  
  # POST v1/complaints/1/accept
  # POST v1/complaints/1/accept.json
  def accept
    @complaint.accepted!
    @complaint.complainable.complaint_accept(complaint_answer_params[:option])
    response = { 'status': 'complaint accepted' }
    render status: :ok, json: response
  end
  
  # POST /v1/complaints/1/treat_related
  # POST /v1/complaints/1/treat_related.json
  def treat_related
    parameters = complaint_answer_params
    user = User.find(params[:id])
    parameters[:complaints].each do | complaints_item |
      complaint = Complaint.find(complaints_item[:id])

      if  user != complaint.complainable
        response = {'error': I18n.t("error.wrong_arguments", param: "id #{complaint.complainable_id}")}
        render status: :bad_request, json: response
      end
      
      if complaints_item[:state] == "accept"
        complaint.accepted!
      elsif complaints_item[:state] == "reject"
        complaint.rejected!
      else
        response = {'error': I18n.t("error.wrong_arguments", param: "state")}
        render status: :bad_request, json: response
      end
    end

    if parameters[:option] != nil
      user.complaint_accept(parameters[:option])
    end

    render status: :ok
  end

  private

  def deleted_resource
    Complaint
  end

  def complaint_creation_params
    params.require(:complaint).permit(:user_id, :description,:complainable_id, :complaint_reason_id, :complainable_type)
  end

  def complaint_answer_params
    params.permit(:id, :option, :complaints=> [:id, :state])    
  end

  def set_complaint
    @complaint ||= Complaint.where(id: params[:id]).first

    render status: :not_found if @complaint.blank?

    @complaint
  end
end
