
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::ItemsController < ApplicationController
  include ::Paginator
  include ::ItemsFilter

  before_action :set_item, only: [:show, :update, :destroy]
  before_action :authenticate_user!, except: [:index, :show]
  before_action :authorize!, only: [:update, :destroy]

  def index
    items = paginate filter_items(policy_scope(Item))
    render json: items
  end

  def show
    render json: @item
  end

  def create
    @item = Item.new(item_params)
    authorize @item

    if @item.save
      render json: @item, status: :created
    else
      render json: @item.errors, status: :unprocessable_entity
    end
  end

  # PUT/PATCH /v1/items/1
  def update
    if @item.update(item_params)
      render json: @item, status: :ok
    else
      render json: @item.errors, status: :unprocessable_entity
    end
  end

  # DELETE /v1/items/1
  def destroy
    if @item.update(state: 2)
      render status: :ok, json: @item
    else
      render json: @item.errors, status: :unprocessable_entity
    end
  end

  private

  def set_item
    @item ||= Item.find_by_id(params[:id])
    if @item.blank?
      render status: :not_found
    end
  end

  def item_params
    params.require(:item).permit(:name, :price, :discount, :description, :state, :item_type, :image, :achievement_id)
  end

  def authorize!
    authorize @item
  end
end
