
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::LearningObjects::AttachmentController < ApplicationController
  include ::Paginator

  before_action :set_objects
  before_action :authenticate_user!, only: [:update, :index]
  before_action :authorize!, only: :destroy
  before_action :authorize_update_attachment!, only: :update

  # GET /learning_objects/:learning_object_id/attachment
  def index
    @attachment = paginate @attachment
    render json: @attachment
  end

  # DELETE /learning_objects/:learning_object_id/attachments/:id
  def destroy
    return render status: :not_found if @learning_object.nil? || @attachment.nil?

    DeleteBitstreamWorker.perform_async(@attachment.id)
    @learning_object.attachments.delete(@attachment)
    if @learning_object.attachment == @attachment
      @learning_object.attachment = nil
      @learning_object.save
      @attachment.destroy
      @learning_object.reload
      @learning_object.default_attachment
    else
      @learning_object.save
      @attachment.destroy
    end

    render status: :ok
  end

  # PUT /learning_objects/:learning_object_id/attachments/:id
  def update
    return render status: :not_found if @learning_object.nil? || @attachment.nil?
    if @attachment.update(update_params)
      render json: @attachment, status: :ok
    else
      render status: :unprocessable_entity
    end
  end

  private

  def attachment_params
    params.permit(:learning_object_id, :id)
  end

  def update_params
    params.permit(:learning_object_attachment_id, :learning_object_attachment_id_son, :infohash)
  end

  def set_objects
    if current_user.try(:is_admin?)
      @learning_object = LearningObject.unscoped.where(id: attachment_params[:learning_object_id]).first
    else
      @learning_object = LearningObject.where(id: attachment_params[:learning_object_id]).first
    end

    render status: :not_found if @learning_object.blank?

    if ( attachment_params[:id].blank? )
      @attachment = @learning_object.attachments
    else
      @attachment = LearningObject::Attachment.where(id: attachment_params[:id]).first
    end

    render status: :not_found if @attachment.blank?
  end

  def authorize!
    return render status: :unauthorized unless @learning_object.attachments.include? @attachment

    authorize(@learning_object, :destroy?)
  end

  def authorize_update_attachment!
    return render status: :unauthorized unless @learning_object.attachments.include? @attachment

    authorize(@learning_object, :add_infohash?)
    authorize(@learning_object, :add_attachment_id?)
  end

end
